#include <stdio.h>

__global__ void WriteIndex(int* a, int n)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    if (i < n) {
        a[i] = i;
    }
}

int main()
{
    // Init
    int N = 16;
    int* h_a = new int[N]; // on host (CPU)
    int* d_a;              // on device (GPU)
    cudaMalloc(&d_a, N * sizeof(int));
    
    // Measure the elapsed time
    cudaEvent_t start, stop;
    float time_ms;
    cudaEventCreate(&start);
    cudaEventCreate(&stop);
    cudaEventRecord(start, 0);

    // Copy data from host to device
    // Do nothing

    // Main
    int threadsPerBlock = 8;
    int blocks = (N + threadsPerBlock - 1) / threadsPerBlock; // round up to int
    WriteIndex<<<blocks, threadsPerBlock>>>(d_a, N);
    
    // Copy data from device to host
    cudaMemcpy(h_a, d_a, N * sizeof(int), cudaMemcpyDeviceToHost);

    // elaspsed time
    cudaEventRecord(stop, 0);
    // Wait until cudaEventRecord is completed. 
    // (Note: cudaEventRecord is an asynchronous function)
    cudaEventSynchronize(stop);
    
    // Display results
    for (int i = 0; i < N; i++) {
        printf("%d\n",h_a[i]);
    }
    cudaEventElapsedTime(&time_ms, start, stop);
    printf("exe time: %f ms\n", time_ms);
    
    // Free memory
    delete [] h_a;
    cudaFree(d_a);
    cudaEventDestroy(start);
    cudaEventDestroy(stop);
    
    return 0;
}

